/* =============================================================================
 * otp_interface.h C OTP interface library header file.
 *  
 * 
 * Created on 2008-11-20
 *            2009-03-12 Add C200(TOTP) API
 *
 * Version:1.0
 *  
 */

#ifndef OTP_INTERFACE_H_
#define OTP_INTERFACE_H_

#ifdef WIN32
//WIN32ƽ̨
typedef unsigned __int64 uint64_t; //֤
#else
//WIN32ƽ̨
#include <stdint.h>

typedef uint64_t authnum_t;
#define __stdcall
#endif

#ifdef __cplusplus
extern		"C" {
#endif

/* ֵ */
#define OTP_SUCCESS					(0x00000000L) //ɹ

#define OTP_ERR_INVALID_PARAMETER	(0x00000001L) //Ч

#define OTP_ERR_CHECK_PWD			(0x00000002L) //֤ʧ

#define OTP_ERR_SYN_PWD				(0x00000003L) //ͬʧ

#define OTP_ERR_REPLAY				(0x000000004) //̬ط



/* =============================================================================
 * Function   : ITSecurity_CheckPwd 
 * Description: OTP ֤ӿ
 * Parameter  : 
 *		authkey			Կ
 *		authnum			֤
 *		authwnd		    ֤Χ, ͨ0-40
 *		otp				Ҫ֤Ķ̬
 *		otplen			Ҫ֤Ķ̬, ͨ6
 *		authresnum		֤ɹ֤
 *
 * return     : 0 - ɹ, ֵΪ.
 */
int __stdcall ITSecurity_CheckPwd (char *authkey, uint64_t authnum, 
				int authwnd, const char *otp, int otplen, 
			 	uint64_t *authresnum);


/* =============================================================================
 * Function   : ITSecurity_PSW_SYN 
 * Description: OTP ֤ͬӿ
 * Parameter  : 
 *		authkey			Կ
 *		authnum			ǰ֤
 *		otp1			Ҫͬĵһ̬
 *		otp1len			Ҫͬĵһ̬, ͨ6
 *		otp2			Ҫͬĵڶ̬
 *		otp2len			Ҫͬĵڶ̬, ͨ6
 *		syncwnd		    ֤ͬΧ, ͨ0-200
 *		authresnum		ͬɹ֤
 *				
 * return     : 0 - ɹ, ֵΪ.
 */
int __stdcall ITSecurity_PSW_SYN (char *authkey, uint64_t authnum, 
				const char *otp1, int otp1len,
				const char *otp2, int otp2len,
			 	int syncwnd, uint64_t *authresnum);


/* =============================================================================
 * Function   : ITSecurity_CheckPwdC200
 * Description: OTP C200(TOTP) ֤ӿ
 * Parameter  : 
 *		authkey			Կ
 *      t               ǰʱUTC Epoch
 *      t0              ʼοʱUTC Epoch(ĬΪ0)
 *      x               TOTP仯(ĬΪ60)
 *      drift           Ưƴ
 *      authwnd		    ֤Χ, ͨ0-20
 *      lastsucc        ǰһ֤ɹUTC Epoch(ΪֹطŹ)
 *      otp				Ҫ֤Ķ̬
 *      otplen			Ҫ֤Ķ̬, ͨ6
 *      currsucc		֤ɹUTC Epoch
 *      currdft         ֤ɹĵǰƯƴ
 *
 * return     : 0 - ɹ, ֵΪ.
 */
int __stdcall ITSecurity_CheckPwdC200(char *authkey, uint64_t t, uint64_t t0, 
        unsigned int x, int drift, int authwnd, uint64_t lastsucc, 
        const char *otp, int otplen, uint64_t *currsucc, int *currdft);

/* =============================================================================
 * Function   : ITSecurity_PSW_SYNC200
 * Description: OTP C200(TOTP) ͬӿ
 * Parameter  : 
 *		authkey			Կ
 *      t               ǰʱUTC Epoch
 *      t0              ʼοʱUTC Epoch(ĬΪ0)
 *      x               TOTP仯(ĬΪ60)
 *      drift           Ưƴ
 *      syncwnd         ͬΧ, ͨ0-20
 *      lastsucc        ǰһ֤ɹUTC Epoch(ΪֹطŹ)
 *      otp1            Ҫͬĵһ̬
 *      otp1len			Ҫͬĵһ̬, ͨ6
 *      otp2            Ҫͬĵڶ̬
 *      otp2len         Ҫͬĵڶ̬, ͨ6
 *      currsucc		֤ɹUTC Epoch
 *      currdft         ֤ɹĵǰƯƴ
 *
 * return     : 0 - ɹ, ֵΪ.
 */
int __stdcall ITSecurity_PSW_SYNC200(char *authkey, uint64_t t, uint64_t t0, 
        unsigned int x, int drift, int syncwnd, uint64_t lastsucc, 
        const char *otp1, int otp1len, const char *otp2, int otp2len, 
        uint64_t *currsucc, int *currdft);


/* =============================================================================
 * Function   : ET_CheckPwdz201
 * Description: OTP C201(TOTP) ֤ӿ
 * Parameter  : 
 *		authkey			Կ
 *      t               ǰʱUTC Epoch
 *      t0              ʼοʱUTC Epoch(ĬΪ0)
 *      x               TOTP仯(ĬΪ60)
 *      drift           Ưƴ
 *      authwnd		    ֤Χ, ͨ0-20
 *      lastsucc        ǰһ֤ɹUTC Epoch(ΪֹطŹ)
 *      otp				Ҫ֤Ķ̬
 *      otplen			Ҫ֤Ķ̬, ͨ6
 *      currsucc		֤ɹUTC Epoch
 *      currdft         ֤ɹĵǰƯƴ
 *
 * return     : 0 - ɹ, ֵΪ.
 */
int __stdcall ET_CheckPwdz201(char *authkey, uint64_t t, uint64_t t0, 
    unsigned int x, int drift, int authwnd, uint64_t lastsucc, 
    const char *otp, int otplen, uint64_t *currsucc, int *currdft);

/* =============================================================================
 * Function   : ET_Syncz201
 * Description: OTP C201(TOTP) ͬӿ
 * Parameter  : 
 *		authkey			ԿѾܹģҪн
 *      t               ǰʱUTC Epoch
 *      t0              ʼοʱUTC Epoch(ĬΪ0)
 *      x               TOTP仯(ĬΪ60)
 *      drift           Ưƴ
 *      syncwnd         ͬΧ, ͨ0-20
 *      lastsucc        ǰһ֤ɹUTC Epoch(ΪֹطŹ)
 *      otp1            Ҫͬĵһ̬
 *      otp1len			Ҫͬĵһ̬, ͨ6
 *      otp2            Ҫͬĵڶ̬
 *      otp2len         Ҫͬĵڶ̬, ͨ6
 *      currsucc		֤ɹUTC Epoch
 *      currdft         ֤ɹĵǰƯƴ
 *
 * return     : 0 - ɹ, ֵΪ.
 */
int __stdcall ET_Syncz201(char *authkey, uint64_t t, uint64_t t0, 
        unsigned int x, int drift, int syncwnd, uint64_t lastsucc, 
        const char *otp1, int otp1len, const char *otp2, int otp2len, 
        uint64_t *currsucc, int *currdft);

#ifdef __cplusplus
}
#endif

#endif /* OTP_INTERFACE_H_*/

