/* =============================================================================
 * php_otpserver.c
 *                      - OTP server library PHP interface implement file.
 *  
 * OTP c100 | OTP c200
 * Copyright (C) 2007-2009 Feitian Technologies Co., Ltd
 * http://www.FTsafe.com
 *
 * Created on 2008-10-07, Last version: 2.0
 *
 * Version:2.0
 *  
 */

#define _USE_32BIT_TIME_T 1

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef WIN32
#    if defined(ZEND_EXPORTS)
#        undef ZEND_EXPORTS
#    endif
#    undef ZEND_DEBUG
#else
#    define PHP_WIN32
#    define ZEND_WIN32
#endif

#define FALSE 0

#ifdef WIN32
#define ZTS                    // Must define ZTS (Zend Thread Safety)
#endif
         
#ifdef THREAD_SAFETY          
#define ZTS                    // define ZTS for Zend Thread Safety
#define PTHREADS			   // Thread Safety type TSRM\TSRM.h
#endif

#define PHP_REGEX_H            // Skip php_regex.h: '_off_t' not defined anywhere
#define COMPILE_DL_PHPVERIFY
#define ZEND_DEBUG    FALSE    // Must be FALSE

#include "php.h"
#include "ext/standard/info.h"
#include "php_otpverify.h"
#include "otp_interface.h"

#define		MAX_FPATH		256

zend_function_entry etotpverify_functions[] =
{
	ZEND_FE(itsecurity_checkpwd, NULL)
	ZEND_FE(itsecurity_psw_syn, NULL)
	ZEND_FE(itsecurity_checkpwdc200, NULL)
	ZEND_FE(itsecurity_psw_sync200, NULL)
	ZEND_FE(et_checkpwdz201, NULL)
	ZEND_FE(et_syncz201, NULL)

	{NULL, NULL, NULL}
};

zend_module_entry etotpverify_module_entry =
{
	 STANDARD_MODULE_HEADER,
	 PHP_OTP_VERIFY_NAME,
	 etotpverify_functions,
	 NULL,
	 NULL,
	 NULL,
	 NULL,
	 PHP_MINFO(etotpverify),
	 PHP_OTP_VERIFY_VERSION,
	 STANDARD_MODULE_PROPERTIES
};


#ifdef COMPILE_DL_PHPVERIFY
ZEND_GET_MODULE(etotpverify)
#endif


PHP_MINFO_FUNCTION(etotpverify)   
{   
	 php_info_print_table_start();

	 php_info_print_table_header(2, PHP_OTP_VERIFY_NAME, "enabled");

	 php_info_print_table_row(2, "Version", PHP_OTP_VERIFY_VERSION);

	 php_info_print_table_end();   
} 


/* =============================================================================
 * Function   : itsecurity_checkpwd 
 * Description: OTP ֤ӿ
 * Parameter  : 
 *		authkey			Կ
 *		authnum			֤
 *		authwnd		    ֤Χ, ͨ0-40
 *		otp				Ҫ֤Ķ̬
 *		otplen			Ҫ֤Ķ̬, ͨ6
 *		authresnum		֤ɹ֤
 *
 * return     : 0 - ɹ, ֵΪ.
 */


ZEND_FUNCTION(itsecurity_checkpwd)
{
	int iRet = OTP_SUCCESS;
	
	/* PHP return param. */
	zval *zretnum;
	uint64_t authresnum = 0;
	/* internal var. */
	char *key, *otp;
	long key_len, otp_len, lotplen, lauthnum, lauthwnd = 0;
	
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sllslz",
								&key, &key_len,
								&lauthnum, &lauthwnd,
								&otp, &otp_len,
								&lotplen, &zretnum)
								== FAILURE) 
	{
		/* invalid parameters */
		RETURN_LONG(OTP_ERR_INVALID_PARAMETER);
	}
	
	/* check point */
// 	if (!PZVAL_IS_REF(zretnum)) 
// 	{
// 		zend_error(E_WARNING, "Parameter(s) wasn't passed by reference.");
// 	}
	

	/* call native function */
	iRet = ITSecurity_CheckPwd(key, (uint64_t)lauthnum, (int)lauthwnd, 
					otp, (int)lotplen, &authresnum);
	
	/* convert */
	if (iRet == OTP_SUCCESS)
	{
		ZVAL_LONG(zretnum, authresnum);
	}
	
	RETURN_LONG(iRet);

}


/* =============================================================================
 * Function   : itsecurity_psw_syn 
 * Description: OTP ֤ͬӿ
 * Parameter  : 
 *		authkey			Կ
 *		authnum			ǰ֤
 *		otp1			Ҫͬĵһ̬
 *		otp1len			Ҫͬĵһ̬, ͨ6
 *		otp2			Ҫͬĵڶ̬
 *		otp2len			Ҫͬĵڶ̬, ͨ6
 *		syncwnd		    ֤ͬΧ, ͨ0-200
 *		authresnum		ͬɹ֤
 *				
 * return     : 0 - ɹ, ֵΪ.
 */

ZEND_FUNCTION(itsecurity_psw_syn)
{
	int iRet = OTP_SUCCESS;
	
	/* PHP return param. */
	zval *zretnum;
	uint64_t authresnum = 0;
	/* internal var. */
	char *key, *otp1, *otp2;
	long key_len, otp1_len, otp2_len;
	long lotp1len, lotp2len, lauthnum, lsyncwnd = 0;
	
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "slslsllz",
								&key, &key_len,
								&lauthnum,
								&otp1, &otp1_len, &lotp1len,
								&otp2, &otp2_len, &lotp2len,
								&lsyncwnd, &zretnum)
								== FAILURE) 
	{
		/* invalid parameters */
		RETURN_LONG(OTP_ERR_INVALID_PARAMETER);
	}
	
	/* check point */
// 	if (!PZVAL_IS_REF(zretnum)) 
// 	{
// 		zend_error(E_WARNING, "Parameter(s) wasn't passed by reference.");
// 	}
	
	
	/* call native function */
	iRet = ITSecurity_PSW_SYN(key, (uint64_t)lauthnum, 
					otp1, (int)lotp1len,
					otp2, (int)lotp2len,
					(int)lsyncwnd, &authresnum);
	
	/* convert */
	if (iRet == OTP_SUCCESS)
	{
		ZVAL_LONG(zretnum, authresnum);
	}
	
	RETURN_LONG(iRet);
}


/* =============================================================================
 * Function   : itsecurity_checkpwdc200
 * Description: OTP C200(TOTP) ֤ӿ
 * Parameter  : 
 *		authkey			Կ
 *      t               ǰʱUTC Epoch
 *      t0              ʼοʱUTC Epoch(ĬΪ0)
 *      x               TOTP仯(ĬΪ60)
 *      drift           Ưƴ
 *      authwnd		    ֤Χ, ͨ0-20
 *      lastsucc        ǰһ֤ɹUTC Epoch(ΪֹطŹ)
 *      otp				Ҫ֤Ķ̬
 *      otplen			Ҫ֤Ķ̬, ͨ6
 *      currsucc		֤ɹUTC Epoch
 *      currdft         ֤ɹĵǰƯƴ
 *
 * return     : 0 - ɹ, ֵΪ.
 */
ZEND_FUNCTION(itsecurity_checkpwdc200)
{
	int iRet = OTP_SUCCESS;
	
	/* PHP return param. */
	zval *zcurrsucc, *zcurrdft;
	uint64_t currsucc = 0;
	int currdft = 0;

	/* internal var. */
	char *key, *otp;
	long key_len, otp_len;
	long lt, lt0, lx, ldrift, lauthwnd = 0, llastsucc, lotplen;
	
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sllllllslzz",
								&key, &key_len,
								&lt, &lt0, &lx, &ldrift,
								&lauthwnd, &llastsucc,
								&otp, &otp_len,
								&lotplen, &zcurrsucc, &zcurrdft)
								== FAILURE) 
	{
		/* invalid parameters */
		RETURN_LONG(OTP_ERR_INVALID_PARAMETER);
	}
	
	/* check point */
// 	if (!PZVAL_IS_REF(zcurrsucc) || !PZVAL_IS_REF(zcurrdft)) 
// 	{
// 		zend_error(E_WARNING, "Parameter(s) wasn't passed by reference.");
// 	}
	
	
	/* call native function */
	iRet = ITSecurity_CheckPwdC200(key, (uint64_t)lt, (uint64_t)lt0,
				(unsigned int)lx, (int)ldrift, 
				(int)lauthwnd, (uint64_t)llastsucc,
				otp, (int)lotplen, 
				&currsucc, &currdft);
	
	/* convert */
	if (iRet == OTP_SUCCESS)
	{
		ZVAL_LONG(zcurrsucc, currsucc);	
		ZVAL_LONG(zcurrdft, currdft);	
	}
	
	RETURN_LONG(iRet);
}


/* =============================================================================
 * Function   : itsecurity_psw_sync200
 * Description: OTP C200(TOTP) ͬӿ
 * Parameter  : 
 *		authkey			Կ
 *      t               ǰʱUTC Epoch
 *      t0              ʼοʱUTC Epoch(ĬΪ0)
 *      x               TOTP仯(ĬΪ60)
 *      drift           Ưƴ
 *      syncwnd         ͬΧ, ͨ0-20
 *      lastsucc        ǰһ֤ɹUTC Epoch(ΪֹطŹ)
 *      otp1            Ҫͬĵһ̬
 *      otp1len			Ҫͬĵһ̬, ͨ6
 *      otp2            Ҫͬĵڶ̬
 *      otp2len         Ҫͬĵڶ̬, ͨ6
 *      currsucc		֤ɹUTC Epoch
 *      currdft         ֤ɹĵǰƯƴ
 *
 * return     : 0 - ɹ, ֵΪ.
 */
ZEND_FUNCTION(itsecurity_psw_sync200)
{
	int iRet = OTP_SUCCESS;
	
	/* PHP return param. */
	zval *zcurrsucc, *zcurrdft;
	uint64_t currsucc = 0;
	int currdft = 0;
	
	/* internal var. */
	char *key, *otp1, *otp2;
	long key_len, otp1_len, otp2_len;
	long lt, lt0, lx, ldrift, lsyncwnd, llastsucc, lotp1len, lotp2len;
	
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sllllllslslzz",
								&key, &key_len,
								&lt, &lt0, &lx, &ldrift,
								&lsyncwnd, &llastsucc,
								&otp1, &otp1_len, &lotp1len,
								&otp2, &otp2_len, &lotp2len,
								&zcurrsucc, &zcurrdft)
								== FAILURE) 
	{
		/* invalid parameters */
		RETURN_LONG(OTP_ERR_INVALID_PARAMETER);
	}
	
	/* check point */
// 	if (!PZVAL_IS_REF(zcurrsucc) || !PZVAL_IS_REF(zcurrdft)) 
// 	{
// 		zend_error(E_WARNING, "Parameter(s) wasn't passed by reference.");
// 	}
	
	
	/* call native function */
	iRet = ITSecurity_PSW_SYNC200(key, (uint64_t)lt, (uint64_t)lt0,
		(unsigned int)lx, (int)ldrift, 
		(int)lsyncwnd, (uint64_t)llastsucc,
		otp1, (int)lotp1len,
		otp2, (int)lotp2len,
		&currsucc, &currdft);
	
	/* convert */
	if (iRet == OTP_SUCCESS)
	{
		ZVAL_LONG(zcurrsucc, currsucc);	
		ZVAL_LONG(zcurrdft, currdft);	
	}
	
	RETURN_LONG(iRet);
}

/* =============================================================================
 * Function   : et_checkpwdz201
 * Description: OTP C201(TOTP) ֤ӿ
 * Parameter  : 
 *		authkey			Կ
 *      t               ǰʱUTC Epoch
 *      t0              ʼοʱUTC Epoch(ĬΪ0)
 *      x               TOTP仯(ĬΪ60)
 *      drift           Ưƴ
 *      authwnd		    ֤Χ, ͨ0-20
 *      lastsucc        ǰһ֤ɹUTC Epoch(ΪֹطŹ)
 *      otp				Ҫ֤Ķ̬
 *      otplen			Ҫ֤Ķ̬, ͨ6
 *      currsucc		֤ɹUTC Epoch
 *      currdft         ֤ɹĵǰƯƴ
 *
 * return     : 0 - ɹ, ֵΪ.
 */
ZEND_FUNCTION(et_checkpwdz201)
{
	int iRet = OTP_SUCCESS;
	
	/* PHP return param. */
	zval *zcurrsucc, *zcurrdft;
	uint64_t currsucc = 0;
	int currdft = 0;

	/* internal var. */
	char *key, *otp;
	long key_len, otp_len;
	long lt, lt0, lx, ldrift, lauthwnd = 0, llastsucc, lotplen;
	
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sllllllslzz",
								&key, &key_len,
								&lt, &lt0, &lx, &ldrift,
								&lauthwnd, &llastsucc,
								&otp, &otp_len,
								&lotplen, &zcurrsucc, &zcurrdft)
								== FAILURE) 
	{
		/* invalid parameters */
		RETURN_LONG(OTP_ERR_INVALID_PARAMETER);
	}
	
	/* check point */
// 	if (!PZVAL_IS_REF(zcurrsucc) || !PZVAL_IS_REF(zcurrdft)) 
// 	{
// 		zend_error(E_WARNING, "Parameter(s) wasn't passed by reference.");
// 	}
	
	/* call native function */
	iRet = ET_CheckPwdz201(key, (uint64_t)lt, (uint64_t)lt0,
				(unsigned int)lx, (int)ldrift, 
				(int)lauthwnd, (uint64_t)llastsucc,
				otp, (int)lotplen, 
				&currsucc, &currdft);
	
	/* convert */
	if (iRet == OTP_SUCCESS)
	{
		ZVAL_LONG(zcurrsucc, currsucc);	
		ZVAL_LONG(zcurrdft, currdft);	
	}
	
	RETURN_LONG(iRet);
}


/* =============================================================================
 * Function   : et_syncz201
 * Description: OTP C201(TOTP) ͬӿ
 * Parameter  : 
 *		authkey			Կ
 *      t               ǰʱUTC Epoch
 *      t0              ʼοʱUTC Epoch(ĬΪ0)
 *      x               TOTP仯(ĬΪ60)
 *      drift           Ưƴ
 *      syncwnd         ͬΧ, ͨ0-20
 *      lastsucc        ǰһ֤ɹUTC Epoch(ΪֹطŹ)
 *      otp1            Ҫͬĵһ̬
 *      otp1len			Ҫͬĵһ̬, ͨ6
 *      otp2            Ҫͬĵڶ̬
 *      otp2len         Ҫͬĵڶ̬, ͨ6
 *      currsucc		֤ɹUTC Epoch
 *      currdft         ֤ɹĵǰƯƴ
 *
 * return     : 0 - ɹ, ֵΪ.
 */
ZEND_FUNCTION(et_syncz201)
{
	int iRet = OTP_SUCCESS;
	
	/* PHP return param. */
	zval *zcurrsucc, *zcurrdft;
	uint64_t currsucc = 0;
	int currdft = 0;
	
	/* internal var. */
	char *key, *otp1, *otp2;
	long key_len, otp1_len, otp2_len;
	long lt, lt0, lx, ldrift, lsyncwnd, llastsucc, lotp1len, lotp2len;
	
	
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sllllllslslzz",
								&key, &key_len,
								&lt, &lt0, &lx, &ldrift,
								&lsyncwnd, &llastsucc,
								&otp1, &otp1_len, &lotp1len,
								&otp2, &otp2_len, &lotp2len,
								&zcurrsucc, &zcurrdft)
								== FAILURE) 
	{
		/* invalid parameters */
		RETURN_LONG(OTP_ERR_INVALID_PARAMETER);
	}
	
	/* check point */
// 	if (!PZVAL_IS_REF(zcurrsucc) || !PZVAL_IS_REF(zcurrdft)) 
// 	{
// 		zend_error(E_WARNING, "Parameter(s) wasn't passed by reference.");
// 	}
	
	/* call native function */
	iRet = ET_Syncz201(key, (uint64_t)lt, (uint64_t)lt0,
		(unsigned int)lx, (int)ldrift, 
		(int)lsyncwnd, (uint64_t)llastsucc,
		otp1, (int)lotp1len,
		otp2, (int)lotp2len,
		&currsucc, &currdft);
	
	/* convert */
	if (iRet == OTP_SUCCESS)
	{
		ZVAL_LONG(zcurrsucc, currsucc);	
		ZVAL_LONG(zcurrdft, currdft);	
	}
	
	RETURN_LONG(iRet);
}
